﻿// ---------------------------------------------------------------------------------
// 
// Sample code is copyright (c) Education Software Solutions Ltd 2021
// 
// Education Software Solutions (Hereafter 'ESS') provide sample code to 
// ESS Technical Integrators (Hereafter 'TIs') under consultancy 
// or as part of our TI programme to assist in the production of applications and services
// to benefit mutual customers.
// 
// Sample code is supplied by the ESS TI Team and may be augmented / changed at any time.
//
// ESS grants non exclusive rights to use the code on an 'as is' basis to contracted TIs 
// for the purpose of interfacing with ESS Products.  Code may be used in whole or in part 
// for the TIs products. Rights are subject to the restrictions below.
//
// This code may not be used for commercial purposes without a current TI agreement   
// in place with ESS.  The use of any sample code may incur liabilities for TI contractual 
// fees.  
//
// If the code is provided to eductational establishments for their own use then further 
// consent from ESS must be obtained before the code can be re-used for commercial gain.
//
// No warranty is provided . This header must be retained within the code and Copyright acknowleged as
// Parts (c) Education Software Solutions Ltd 2021 in version info.
//
// Code samples may be used to support other partners and no exclusivity is implied herein
//
// ---------------------------------------------------------------------------------

using System;
using System.Net.Http;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Runtime.Intrinsics;
using System.Xml.Schema;
using PostitCommon;
using PostitCore;
using SIMS8V3Models;

namespace DemoCode
{
    class Program
    {
        public static CallInfo callInfo = new CallInfo();
        public static HttpClient httpClient = new HttpClient();
        static public Dictionary<Guid, string> AttCodeLookup = new Dictionary<Guid, string>();

        static void Main(string[] args)
        {
            callInfo.SetCredentialsPostV2Upgrade(httpClient);
            if (callInfo.bearerToken.Token != "")
            {
                DoTest();
            }

        }
        public static string DoCall(string URL, string BearerToken, HttpClient _httpClient, string SubscriptionKey)
        {
            string resp = "";
            try
            {
                PostitCommon.Call c177 = new PostitCommon.Call(URL);
                // Execute the call
                // Passes in the bearer token and the subscription key
                resp = c177.Execute(BearerToken, _httpClient, SubscriptionKey);
            }
            catch
            {

            }
            return resp;
        }
        public static void DoTest()
        {

            // Set up the log file or the demo goes wrong very wierdly!


            PostitCommon.Log.LogFile = CallInfo.LogFile;

            #region Demo Call - verbose for attendance Lookups
            // Create a call
            // Simply need to build the URL.
            // <Base> ... Put this in to configuration data - It will change when DeX comes in
            // but it will simply be a new base url the rest remains the same in theory.
            // <service>   Attendance
            // <Version> V3 - All are V3 but odd ones may come back with a different version if they
            // give you a URL e.g. photos
            // <Information Domain> e.g. AttendanceRecord
            // <Base>/<Service>/<Version>/<Information Domain>
            // Note that this is different to before

            string URL = CallInfo.v3APIs + "Attendance/v3/AttendanceLookups";
            string resp = DoCall(URL, callInfo.bearerToken.Token, httpClient, callInfo.arq.OCP_APIM_Key);
            // Error handling should be better 
            List<TAttendanceLookups> AttendanceCodes = new List<TAttendanceLookups>();
            List<TAttendanceLookups> StatutoryAttendanceCodes = new List<TAttendanceLookups>();

            try
            {
                // If it looks cushtie then we should be able to use the template classes
                // to deserialise the response.

                // Easy peasy lemon squeezy.

                var _ALUS = Newtonsoft.Json.JsonConvert.DeserializeObject<ODataResponse<TAttendanceLookups>>(resp);

                // You probably want a list of statutory attendance codes and attendance codes.
                foreach (TAttendanceLookups l in _ALUS.Value)
                {
                    if (l.LookupType == "AttendanceCode")
                    {
                        AttendanceCodes.Add(l);
                    }
                    if (l.LookupType == "StatutoryAttendanceCode")
                    {
                        StatutoryAttendanceCodes.Add(l);
                    }
                }
                // Now use the rest of the classes to load what you need and munge the data to where 
                // you need it to be.
            }
            catch
            {
                Log.Write("Error: " + resp);
            }
            Dump.File(resp, Path.Combine(CallInfo.Folder, "AttLookups.json"), "Save attendance Lookups");
            #endregion
            #region Better way to get attendance codes.
            URL = CallInfo.v3APIs + "Attendance/v3/AttendanceLookups?$filter=LookupType eq 'AttendanceCode'";
            resp = DoCall(URL, callInfo.bearerToken.Token, httpClient, callInfo.arq.OCP_APIM_Key);
            // Error handling should be better 
            List<TAttendance_Code> AttendanceCodes2 = new List<TAttendance_Code>();
            try
            {
                var _ALUS = Newtonsoft.Json.JsonConvert.DeserializeObject<ODataResponse<TAttendance_Code>>(resp);
                AttendanceCodes2 = _ALUS.Value;
            }
            catch
            {
                Log.Write("Error: " + resp);
            }
            #endregion

            #region Pastoral Group - NC Year
            // "PastoralGroupType eq 'SchoolNCYear' could filter
            URL = CallInfo.v3APIs + "/PastoralGroup/V3/PastoralGroupDefinitions?$filter=PastoralGroupType eq 'SchoolNCYear'";
            resp = DoCall(URL, callInfo.bearerToken.Token, httpClient, callInfo.arq.OCP_APIM_Key);
            // Error handling should be better 
            Guid firstNCYear = Guid.Empty;
            try
            {
                var _PG = Newtonsoft.Json.JsonConvert.DeserializeObject<ODataResponse<TPastoralGroupDefinition>>(resp);
                if (_PG.Value.Count !=0)
                {
                    firstNCYear = _PG.Value[0].ExternalID;
                }
             }
            catch (Exception ex)
            { 
                Log.Write("Error: " + resp);
            }
            #endregion
            #region Attendance Records
            // "PastoralGroupType eq 'SchoolNCYear' could filter
            URL = CallInfo.v3APIs + "Attendance/V3/AttendanceRecords?$filter=AttendableEventStart eq 2014-02-13T13:00:00Z";
            resp = DoCall(URL, callInfo.bearerToken.Token, httpClient, callInfo.arq.OCP_APIM_Key);
            // Error handling should be better 
            try
            {
                var _AR = Newtonsoft.Json.JsonConvert.DeserializeObject<ODataResponse<TAttendanceRecord>>(resp);
            }
            catch (Exception ex)
            {
                Log.Write("Error: " + resp);
            }
            #endregion

            #region Pastoral Group - NC Year Members
            // "PastoralGroupType": "SchoolNCYear" could filter
            URL = CallInfo.v3APIs + "/PastoralGroup/V3/PastoralGroupMemberships?$filter=PastoralGroup/ExternalID eq " + firstNCYear.ToString()  ;
            resp = DoCall(URL, callInfo.bearerToken.Token, httpClient, callInfo.arq.OCP_APIM_Key);
            // Error handling should be better 
            try
            {
                var _Members = Newtonsoft.Json.JsonConvert.DeserializeObject<ODataResponse<TPastoralGroupMembers>>(resp);
            }
            catch (Exception ex)
            {
                Log.Write("Error: " + resp);
            }
            #endregion
        }
    }
}
