﻿// ---------------------------------------------------------------------------------
// 
// Sample code is copyright (c) Education Software Solutions Ltd 2021
// 
// Education Software Solutions (Hereafter 'ESS') provide sample code to 
// ESS Technical Integrators (Hereafter 'TIs') under consultancy 
// or as part of our TI programme to assist in the production of applications and services
// to benefit mutual customers.
// 
// Sample code is supplied by the ESS TI Team and may be augmented / changed at any time.
//
// ESS grants non exclusive rights to use the code on an 'as is' basis to contracted TIs 
// for the purpose of interfacing with ESS Products.  Code may be used in whole or in part 
// for the TIs products. Rights are subject to the restrictions below.
//
// This code may not be used for commercial purposes without a current TI agreement   
// in place with ESS.  The use of any sample code may incur liabilities for TI contractual 
// fees.  
//
// If the code is provided to eductational establishments for their own use then further 
// consent from ESS must be obtained before the code can be re-used for commercial gain.
//
// No warranty is provided . This header must be retained within the code and Copyright acknowleged as
// Parts (c) Education Software Solutions Ltd 2021 in version info.
//
// Code samples may be used to support other partners and no exclusivity is implied herein
//
// ---------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;
using Newtonsoft.Json;

/// <summary>
/// Copyright Capita ESS 2020
/// This code is intended for the use of ESS technical integrators
/// Code is provided without warranty
/// </summary>
namespace SIMS8V3Models
{
    [JsonObject(MemberSerialization.OptIn)]
    public class TAttendanceRecordForPut
    {
        [JsonProperty]
        public Guid ExternalID;
        [JsonProperty]
        public TLearnerIdentifier Attendee;
        [JsonProperty]
        public TAttendanceStatutorySessions Event;
        [JsonProperty]
        public TAttendance_Code AttendanceCode;
        [JsonProperty]
        public TAttendanceQualifier Qualifier;
        public string SingleSaveFormat()
        {
            // WARNING - This should simply be a toString on an attendance record
            StringBuilder b = new StringBuilder();
            b.AppendLine("{");
            //b.AppendLine("  \"Present\": " + Present.ToString().ToLower() + ",");
            //b.AppendLine("  \"Absent\": " + Absent.ToString().ToLower() + ",");
            // The following 3 items should not be posted but may accept nulls or well formed values
            // most likely is that worst can, an error occurs
            //b.AppendLine("  \"AttendableEventStart\": \"" + ToJSONDateTimeFromString(AttendableEventStart) + "\",");
            //b.AppendLine("  \"RecordedOn\": \"" + ToJSONDateTime(RecordedOn) + "\",");
            //b.AppendLine("  \"RecordedBy\": \"" + Attendee.ExternalID.ToString() + "\",");
            b.Append(Event.ToString());
            b.AppendLine(Attendee.ToAttendanceMarkInsert());
            b.Append(AttendanceCode.ToAttendanceCodeInsert());
            b.Append(Qualifier.ToString());
            b.AppendLine("}");
            string x = b.ToString();
            return b.ToString();
        }
        public string BatchFormat()
        {

            // Warning - not updated when this went live.

            // Should be a simpel to string
            StringBuilder b = new StringBuilder();
            b.AppendLine("{");
            b.AppendLine("@odata.type\":\"#AttendanceService.Models.OData.AttendanceRecord\",");
            b.AppendLine("\"AttendableEvent\":\"" + Event.ToString() + "\",");
            b.AppendLine("\"SchoolAttendanceCode\":\"" + AttendanceCode.ToString() + "\",");
            b.AppendLine("\"Learner\":\"" + Attendee.ExternalID.ToString() + "\",");
            b.AppendLine("\"Comment\":\"Added for testing batch\",");
            b.AppendLine("\"MinutesLate\":null");
            b.AppendLine("}");
            return b.ToString();
        }
        public static Guid PresentGuid = Guid.Empty;
        public static Guid AbsentGui = Guid.NewGuid();  // Don't make them the same 


    }
    /// <summary>
    /// Note that this class is read only!
    /// Only the data in the base class is needed for a save / put
    /// NB: can't patch attendance marks.
    /// </summary>
    [JsonObject(MemberSerialization.OptIn)]
    public class TAttendanceRecord: TAttendanceRecordForPut
    {

        [JsonProperty]
        public Nullable<bool> Present;
        [JsonProperty]
        public Nullable<bool> Absent;
        [JsonProperty]
        public TAttendanceRecorder RecordedBy;
        public Nullable<DateTime> RecordedOn;
        [JsonProperty]
        public string ChangeProcessName;
        [JsonProperty]
        public string AttendableEventStart;

        public TAttendanceRecord(Guid ID,
                                 Nullable<bool> _Present,
                                 Nullable<bool> _Absent,
                                 TAttendanceRecorder _RecordedBy,
                                 Nullable<DateTime> _RecordedOn,
                                 string _ChangeProcessName,
                                 string _Start,
                                 TLearnerIdentifier _Learner,
                                 TAttendanceStatutorySessions _Event,
                                 TAttendance_Code _AttCode)
        {
            Present = _Present;
            Absent = _Absent;
            RecordedBy = _RecordedBy;
            RecordedOn = _RecordedOn;
            ChangeProcessName = _ChangeProcessName;
            AttendableEventStart = _Start;
            Attendee = _Learner;
            Event = _Event;
            AttendanceCode = _AttCode;
            Qualifier = new TAttendanceQualifier();
            Qualifier.MinutesLate = null;
            Qualifier.Comments = "Testing";
        }


    }
    public class TAttendanceStatutorySessions : TAttendableEvent
    {

    }
    /// <summary>
    /// Attendance has the concept of attendable events which is simply
    /// an event that someone could attend and have their attendance at the event recorded
    /// in an attendance record.
    /// </summary>
    public class TAttendableEvent
    {
        public Guid ExternalID;
        public string Name;
        public string Description;
        public Nullable<Guid> SessionType;
        public Nullable<DateTime> StartDateTime;
        public Nullable<DateTime> EndDateTime;
        //"Event": {
        //  "ExternalID": "eaea8393-fcb0-4abe-a157-6351136bce07",
        //  "Type": "Attendance.AttendanceSession"
        //}
        /// <summary>
        /// Create an insert for saving an attendance mark
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            StringBuilder b = new StringBuilder();
            b.AppendLine("  \"Event\": {");
            b.AppendLine("    \"ExternalID\": \"" + ExternalID.ToString() + "\",");
            b.AppendLine("    \"Type\": \"Attendance.AttendanceSession\"");
            b.AppendLine("  },");

            return b.ToString();
        }
    }
    /// <summary>
    /// An attendance qualifier is a bunch of additional fields relating to an attendance 
    /// record which may not exist
    /// 
    /// Take note of the ToString because the write back is more exacting than it
    /// perhaps should be.
    /// 
    /// </summary>
    public class TAttendanceQualifier
    {
        public Guid ExternalID;
        public Nullable<int> MinutesLate;
        public string Comments;
        public override string ToString()
        {
            StringBuilder b = new StringBuilder();
            b.AppendLine("  \"Qualifier\": {");
            // You can't save zero minutes late
            if (MinutesLate != null && MinutesLate != 0)
            {
                b.AppendLine("    \"Comment\":\"" + Comments + "\",");
                b.AppendLine("    \"MinutesLate\":" + MinutesLate);
            }
            else
            {
                b.AppendLine("    \"Comment\":\"" + Comments + "\"");

            }
            b.AppendLine("  }");
            return b.ToString();
        }
    }


    /// <summary>
    /// Base class for attendance codes
    /// </summary>
    public class TAttendanceLookups
    {
        public Guid ExternalID;
        public string Code;
        public string Description;
        public string LookupType;
        public TResourceProvider ResourceProvider;
        public Nullable<int> DisplayOrder;
        public Nullable<bool> IsVisible;


    }
    public class TPhysicalCategory : TReferenceClass { }
    public class TStatisticalCategory : TReferenceClass { }

    public class TStatutoryAttendanceCodeRef : TReferenceClass { }
    /// <summary>
    /// Attendance code which is used to find the code and description for
    /// an attendance mark.
    /// </summary>
    public class TStatutoryAttendance_Code : TAttendanceLookups
    {
        public TPhysicalCategory PhysicalCategory;
        public TStatisticalCategory StatisticalCategory;

    }
    public class TAttendance_Code : TAttendanceLookups
    {
        public TStatutoryAttendanceCodeRef StatutoryAttendanceCode;
        /// <summary>
        /// Convert an attendance code in to something that can be saved as part of an 
        /// attendance record
        /// </summary>
        /// <returns></returns>
        public string ToAttendanceCodeInsert()
        {
            StringBuilder b = new StringBuilder();
            b.AppendLine("  \"AttendanceCode\": {");
            b.AppendLine("    \"ExternalID\": \"" + ExternalID.ToString() + "\",");
            b.AppendLine("    \"Type\": \"Attendance.Lookup\"");
            b.AppendLine("  },");

            return b.ToString();
        }

    }
    /// <summary>
    /// The user associated with the process when a mark is updated
    /// Audit
    /// </summary>
    public class TAttendanceRecorder
    {
        public Guid ExternalID;
        public string Type;
    }
}