﻿// ---------------------------------------------------------------------------------
// 
// Sample code is copyright (c) Education Software Solutions Ltd 2021
// 
// Education Software Solutions (Hereafter 'ESS') provide sample code to 
// ESS Technical Integrators (Hereafter 'TIs') under consultancy 
// or as part of our TI programme to assist in the production of applications and services
// to benefit mutual customers.
// 
// Sample code is supplied by the ESS TI Team and may be augmented / changed at any time.
//
// ESS grants non exclusive rights to use the code on an 'as is' basis to contracted TIs 
// for the purpose of interfacing with ESS Products.  Code may be used in whole or in part 
// for the TIs products. Rights are subject to the restrictions below.
//
// This code may not be used for commercial purposes without a current TI agreement   
// in place with ESS.  The use of any sample code may incur liabilities for TI contractual 
// fees.  
//
// If the code is provided to eductational establishments for their own use then further 
// consent from ESS must be obtained before the code can be re-used for commercial gain.
//
// No warranty is provided . This header must be retained within the code and Copyright acknowleged as
// Parts (c) Education Software Solutions Ltd 2021 in version info.
//
// Code samples may be used to support other partners and no exclusivity is implied herein
//
// ---------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;

namespace SIMS8V3Models
{
    #region Reference Classes
    /// <summary>
    /// There are many reference classes - this is to reduce the number of code lines
    /// </summary>
    public abstract class TReferenceClass
    {
        public Guid ExternalID;
        public string Type;
    }
    public abstract class TNotes
    {
        public Guid ExternalID;
        public string Summary;
        public string Note;
        public Nullable<bool> Confidential;
        public List<string> Attachments;
    }
    public abstract class TLookups
    {
        public Guid ExternalID;
        public TResourceProviderRef ResourceProvider;
        public string Description;
        public string Code;
        public Nullable<int> DisplayOrder;
        public Nullable<bool> IsVisible;
        public string LookupType;
        public string ExternalCode;
    }
    #endregion
    #region Learner Registrations
    public class TLearnerRegistrations
    {
        public Guid ExternalID;
        public string AdmissionNumber;
        public string UniquePupilNumber;
        public string UniqueLearnerNumber;
        public string LocalUPN;
        public Nullable<bool> IsEducatedAtAnotherSiteButNotInASchool;
        public List<TLearnerEnrolment> Enrolments;
        public List<TLearnerBoarderStatus> BoarderStatuses;
        public List<TLearnerPartTimeRecord> PartTimeRecords;
        public List<TLearnerUPNHistory> UPNHistory;

    }
    public class TLearnerEnrolment
    {
        public Guid ExternalID;
        public Nullable<DateTime> DateOfAdmission;
        public Nullable<DateTime> DateOfLeaviing;
        public Nullable<DateTime> DestinationStartDate;
        public string Destination;
        public TReasonForLeavingRef ReasonForLeaving;
        public TGroundsForRemovalRef GroundsForRemoval;
        public TDestinationEstablishment DestinationEstablishment;
        public List<TLearnerEnrolmentStatus> EnrolmentStatuses;
    }
    public class TLearnerEnrolmentStatus
    {
        public Guid ExternalID;
        public Nullable<DateTime> StartDate;
        public Nullable<DateTime> EndDate;
        public TEnrolmentStatusRefObj EnrolmentStatus;
    }
    public class TDestinationEstablishment : TReferenceClass { }
    public class TEnrolmentStatusRefObj : TReferenceClass { }
    public class TReasonForLeavingRef : TReferenceClass { }
    public class TGroundsForRemovalRef : TReferenceClass { }
    public class TBoarderStatusRef : TReferenceClass { }
    public class TLearnerBoarderStatus
    {
        public Guid ExternalID;
        public TBoarderStatusRef BoarderStatus;
        public Nullable<DateTime> StartDate;
        public Nullable<DateTime> EndDate;
    }
    public class TLearnerPartTimeRecord
    {
        public Guid ExternalID;
        public Nullable<DateTime> StartDate;
        public Nullable<DateTime> EndDate;
    }
    public class TLearnerUPNHistory
    {
        public Guid ExternalID;
        public Nullable<DateTime> EndDate;
        public string UniquePupilNumber;
    }
    #endregion Learner Registrations
    #region Learner Medical
    public class TLearnerMedical
    {
        public Guid ExternalID;
        public string NHSNumber;
        public Nullable<bool> IsDisabled;
        public Nullable<bool> IsPregnant;
        public List<TLearnerMedicalCondition> MedicalConditions;
        public List<TLearnerDisability> Disabilities;
        public List<TLearnerMedicalEvent> MedicalEvents;
        public List<TLearnerMedicalNote> MedicalNotes;
        public List<TLearnerParamedicalSupport> LearnerParamedicalSupports;
        public TMedicalProviderRef MedicalProviders;
    }
    public class TLearnerMedicalCondition
    {
        public Guid ExternalID;
        public Nullable<DateTime> InformationReceivedDate;
        public Nullable<DateTime> EndDate;
        public List<TLearnerMedicalConditionAttachment> MedicalConditionNotes;
        public TMedicalConditionRef MedicalCondition;
        public string Summary;
    }
    public class TMedicalConditionRef : TReferenceClass { }
    public class TLearnerMedicalConditionAttachment : TReferenceClass { }
    public class TDisabilityTypeRef : TReferenceClass { }
    public class TLearnerDisability
    {
        public Guid ExternalID;
        public Nullable<DateTime> StartDate;
        public Nullable<DateTime> EndDate;
        public string Notes;
        public TDisabilityTypeRef DisabilityType;
    }
    public class TLearnerMedicalEvent
    {
        public Guid ExternalID;
        public Nullable<DateTime> EventDate;
        public Nullable<DateTime> FollowUpDate;
        public TMedicalEventDescriptionRef MedicalEventDescription;
        public TMedicalEventTypeRef MedicalEventType;
        public TLearnerMedicalEventAttachment MedicalEventNotes;
    }
    public class TMedicalEventDescriptionRef : TReferenceClass { }
    public class TMedicalEventTypeRef : TReferenceClass { }
    public class TLearnerMedicalEventAttachment : TReferenceClass { }
    public class TLearnerMedicalNote
    {
        public Guid ExternalID;
        public Nullable<bool> Confidential;
        public string Summary;
        public string Note;
        public List<string> Attachments;
    }
    public class TLearnerParamedicalSupport
    {
        public Guid ExternalID;
        public TParamedicalSupportTypeRef ParamedicalSupportType;
    }
    public class TParamedicalSupportTypeRef : TReferenceClass { }
    public class TMedicalProviderRef : TReferenceClass { }
    #endregion Learner Medical
    #region Learner Groups
    public class TLearnerGroup
    {
        public Guid ExternalID;
        public string GroupType;
        public Nullable<int> DisplayOrder;
        public string ShortName;
        public string FullName;
    }
    #endregion Learner Groups
    #region Learner Lookups
    public class TLearnerLookups : TLookups { }

    #endregion Learner Lookups
    #region Learner Personal
    public class TLearnerPersonal
    {

        /// <summary>
        /// The unique identifier for the Learner
        /// </summary>
        public Guid ExternalID;
        /// <summary>
        /// The known as forename of the learner
        /// </summary>
        public string PreferredForename;
        /// <summary>
        /// The known as or preferred surname of a learner
        /// </summary>
        public string PreferredSurname;
        /// <summary>
        /// Gender description of the learner
        /// </summary>
        public string GenderCategory;
        /// <summary>
        /// The age of the learner in months
        /// </summary>
        public Nullable<DateTime> DateOfBirth;
        public Nullable<DateTime> DateOfDeath;
        public string LegalForename;
        public string LegalMiddleNames;
        public string LegalSurname;
        public string QuickNote;
        public string Photograph;
        public Nullable<bool> BirthCertificateSeen;
        public TGenderRef Gender;
        public TTitleRef Title;
    }
    public class TResourceProviderRef : TReferenceClass { }
    public class TTitleRef : TReferenceClass { }
    public class TGenderRef : TReferenceClass { }
    #endregion Learner Personal
    #region Learner Identifiers
        public class TLearnerIdentifier
        {
            /// <summary>
            /// The unique identifier for the Learner
            /// </summary>
            public Guid ExternalID;
            /// <summary>
            /// The known as forename of the learner
            /// </summary>
            public string Forename;
            /// <summary>
            /// The known as or preferred surname of a learner
            /// </summary>
            public string Surname;
            /// <summary>
            /// Gender description of the learner
            /// </summary>
            public string GenderCategory;
            /// <summary>
            /// The age of the learner in months
            /// </summary>
            public int? AgeMonths;
            public string Photograph;
            public bool? DoNotCommunicate;
            public TLearnerGroup CurrentYearGroup;
            public TLearnerGroup CurrentPrimaryClass;
            public TLearnerGroup CurrentHouse;
            public string OnRollState;
            public DateTime? DateOfApplication;
            public override string ToString()
            {
                return Forename + " " + Surname + " Age: " + AgeMonths.ToString() + " months";
            }
            public string ToAttendanceMarkInsert()
            {
                //"Attendee": {
                //  "ExternalID": "e1d19c5e-1910-4350-8a7e-c4b3ed03daaa",
                //  "Type": "Learner"
                //},
                StringBuilder b = new StringBuilder();
                b.AppendLine("  \"Attendee\": {");
                b.AppendLine("    \"ExternalID\": \"" + ExternalID.ToString() + "\",");
                b.AppendLine("    \"Type\": \"Learner\"");
                b.AppendLine("  },");

                return b.ToString();
            }
        }
    #endregion LearnerIdentifiers
    #region Learner Cultural
    public class TLearnerCultural
    {
        /// <summary>
        /// The unique identifier for the Learner
        /// </summary>
        public Guid ExternalID;
        public Nullable<DateTime> DateOfEntryIntoEWBES;
        public Nullable<bool> PreviouslyEducatedInNEWBES;
        public Nullable<bool> IsTaughtThroughIrishMedium;
        public Nullable<bool> EnglishAsAdditional;
        public TCountryOfBirth CountryOfBirth;
        public TEthnicityRef Ethnicity;
        public TEthnicDataSourceRef EthnicDataSource;
        public TReligionRef Religion;
        public TAsylumSeekerRef AsylumSeeker;
        public TWelshDataSourceRef WelshDataSource;
        public TTaughtWelshRef TaughtWelsh;
        public TSpeaksWelshWithParents SpeaksWelshWithParents;
        public TSpeaksWelshAtHome SpeaksWelshAtHome;
        public TSpeaksWelshRef SpeaksWelsh;
        public TSpeaksWelshWithSiblings SpeaksWelshWithSiblings;
        public TNationalIdentifierRef NationalIdentifier;
        public List<TLearnerLanguageDetail> LearnerLanguageDetails;
        public List<TLearnerNationalities> LearnerNationalities;
        public List<TLearnerProficiencyInNationalLanguage> NationalLanguageProficiencyTests;
        public List<TLearnerPassportDetail> LearnerPassportDetails;
        public List<TLearnerNewcomerPeriod> NewcomerPeriods;
        public TLearnerTravellerDetail LearnerTravellerDetails;
    }

    public class TReligionRef : TReferenceClass { }
    public class TCountryOfBirth : TReferenceClass { }
    public class TEthnicityRef : TReferenceClass { }
    public class TEthnicDataSourceRef : TReferenceClass { }
    public class TAsylumSeekerRef : TReferenceClass { }
    public class TWelshDataSourceRef : TReferenceClass { }
    public class TTaughtWelshRef : TReferenceClass { }
    public class TSpeaksWelshWithParents : TReferenceClass { }
    public class TSpeaksWelshAtHome : TReferenceClass { }
    public class TSpeaksWelshRef : TReferenceClass { }
    public class TSpeaksWelshWithSiblings : TReferenceClass { }
    public class TNationalIdentifierRef : TReferenceClass { }
    public class TLanguageRef : TReferenceClass { }
    public class TLanguageUseRef : TReferenceClass { }
    public class TSourceOfInformation : TReferenceClass { }
    public class TLearnerLanguageDetail
    {
        public Guid ExternalID;
        public TLanguageRef Language;
        public TLanguageUseRef LanguageUse;
        public TSourceOfInformation SourceOfInformation;
    }
    public class TLearnerTravellerDetail
    {
        public Guid ExternalID;
        public Nullable<DateTime> StartDate;
        public Nullable<DateTime> EndDate;
        public TTravellerAccommodationTypeRef TravellerAccommodationType;
        public TTravellerSourceRef TravellerSource;
    }
    public class TTravellerSourceRef : TReferenceClass { }
    public class TTravellerAccommodationTypeRef : TReferenceClass { }
    public class TLearnerNationalities : TReferenceClass { }
    public class TLearnerProficiencyInNationalLanguage : TReferenceClass { }
    public class TLearnerPassportDetail : TReferenceClass { }
    public class TLearnerNewcomerPeriod : TReferenceClass { }
    #endregion Learner Cultural
    #region Learner Dietary
    public class TDietaryNeedRef : TReferenceClass { }
    public class TUKCountryRef : TReferenceClass { }
    public class TFreeMealEligibility
    {
        public Guid ExternalID;
        public Nullable<DateTime> StartDate;
        public Nullable<DateTime> EndDate;
        public Nullable<DateTime> ReviewDate;
        public TUKCountryRef UKCountry;
        public Nullable<bool> Protected;
        public string Notes;
    }
    public class TMealPatternDiningSession : TReferenceClass { }
    public class TMealPatterns
    {
        public Guid ExternalID;
        public Nullable<DateTime> StartDate;
        public Nullable<DateTime> EndDate;
        public List<TMealPatternDiningSession> MealPatternDiningSessions;
    }
    public class TFreeSchoolMilkEligibility
    {
        public Guid ExternalID;
        public Nullable<DateTime> StartDate;
        public Nullable<DateTime> EndDate;
    }
    public class LearnerDietary
    {
        public Guid ExternalID;
        public List<TFreeSchoolMilkEligibility> FreeSchoolMilkEligibilityPeriods;
        public List<TFreeMealEligibility> FreeMealEligibilityPeriods;
        public List<TMealPatterns> MealPatterns;
        public List<TDietaryNeedRef> DietaryNeeds;
    }
    #endregion
    #region Learner SEN

    public class TLearnerSENProvisionType
    {
        public Guid ExternalID;
        public Nullable<DateTime> StartDate;
        public Nullable<DateTime> EndDate;
        public string Comment;
        public List<TSENProvisionAttachment> Notes;
        public TSENProvisionTypeRef SENProvisionType;
    }
    public class TSENAdviceAndAssessmentRef : TReferenceClass { }
    public class TSENGroupingAndSupportRef : TReferenceClass { }
    public class TSENCurriculumAndTeachingMethodRef : TReferenceClass { }
    public class TLearnerSENNeedTypeNote : TNotes { }
    public class TSENNeedTypeRef : TReferenceClass { }
    public class TSENProvisionAttachment : TNotes { }
    public class TSENProvisionTypeRef : TReferenceClass { }
    public class TLearnerSENStatus : TReferenceClass { }
    public class TSubject : TReferenceClass { }
    public class TLearnerMostAble
    {
        public Guid ExternalID;
        public Nullable<DateTime> FromDate;
        public Nullable<DateTime> ToDate;
        public TSubject Subject;
    }

    public class TLearnerSENNeedTypes
    {
        public Guid ExternalID;
        public List<TLearnerSENNeedTypeNote> Notes;
        public TSENNeedTypeRef SENNeedType;
        public Nullable<DateTime> StartDate;
        public Nullable<DateTime> EndDate;
        public string Description;
        public Nullable<int> Rank;
    }

    public class TLearnerSEN
    {
        public Guid ExternalID;
        public string LearningStrategies;
        public TSENGroupingAndSupportRef SENGroupingAndSupport;
        public TSENCurriculumAndTeachingMethodRef SENCurriculumAndTeachingMethod;
        public TSENAdviceAndAssessmentRef SENAdviceAndAssessment;
        public List<TLearnerSENNeedTypes> SENNeeds;
        public List<TLearnerSENProvisionType> SENProvisions;
        public List<TLearnerSENStatus> SENStatuses;
        public List<TLearnerMostAble> MostAbleSubjects;
    }
    #endregion Learner SEN
    #region SEN Lookups
    public class TLearnerSENLookup : TLookups { }
    #endregion SEN Lookup
    #region Learner Contact Details
    public class TTelephoneRef
    {
        public Guid ExternalID;
        public string TelephoneNumber;
        public Nullable<bool> IsFirstPointOfContact;
        public Nullable<bool> UseForAutomatedMessages;
        public string Notes;
        public TLocationTypeRef LocationType;
    }
    public class TEmailRef
    {
        public Guid ExternalID;
        public string EmailAddress;
        public Nullable<bool> IsMainEmail;
        public Nullable<bool> UseForAutomatedMessages;
        public string Notes;
        public TEmailLocationTypeRef LocationType;
    }
    public class TAddress
    {
        public Guid ExternalID;
        public string Apartment;
        public string BuildingNumber;
        public string BuildingName;
        public string Street;
        public string District;
        public string Town;
        public string AdministrativeArea;
        public string Postcode;
        public string CoordinateX;
        public string CoordinateY;
        public string USRN;
        public string UPRN;
        public string AddressIdentifier;
        public Nullable<DateTime> LastValidated;
        public TCountryRef Country;
        public TResourceProviderRef ResourceProvider;
    }
    public class TLearnerContactRelationshipTypeRef : TReferenceClass  { }
    public class TLearnerContactDetail : TReferenceClass {}
    public class TLearnerContactRelationship
    {
        public Guid ExternalID;
        public Nullable<bool> HasCourtOrder;
        public Nullable<bool> HasParentalResponsibility;
        public Nullable<bool> ReceivesCorrespondance;
        public Nullable<bool> ReceivesSchoolReport;
        public Nullable<int> Priority;
        public TLearnerContactRelationshipTypeRef LearnerContactRelationshipType;
        public TLearnerContactDetail LearnerContact;
    }
    public class TLearnerContactDetails
    {
        public Guid ExternalID;
        public string ParentalSalutation;
        public string ParentalAddressee;
        public Nullable<bool> IsMailingPoint;
        public List<TLearnerContactRelationship> LearnerContactRelationships;
        public List<TAddress> Addresses;

        public List<TTelephoneRef> Telephones;
        public List<TEmailRef> Emails;
    }
    public class TLocationTypeRef : TReferenceClass { }

    public class TCountryRef : TReferenceClass { }
    public class TEmailLocationTypeRef : TReferenceClass { }
    #endregion
    #region Contact 
    public class TContactBasicHomeLanguageRef : TReferenceClass { }
    public class TContactBasicOccupationReference : TReferenceClass { }
    public class TContactBasicContactNotes : TNotes { }
    public class TContactDetails
    {
        public Guid ExternalID;
        public string Surname;
        public string MiddleName;
        public Nullable<bool> IsTranslatorRequired;
        public Nullable<bool> ParentalBallot;
        public string Honours;
        public string Salutation;
        public string Addressee;
        public string PlaceOfWork;
        public string JobTitle;
        public TTitleRef Title;
        public TGenderRef Gender;
        public TContactBasicHomeLanguageRef HomeLanguage;
        public TContactBasicOccupationReference Occupation;
        public List<TContactBasicContactNotes> Notes;
    }
    public class TContactContactDetails
    {
        public Guid ExternalID;
        public string Addressee;
        public string Salutation;
        public List<TTelephoneRef> Telephones;
        public List<TEmailRef> Emails;
        public List<TContactContactDetailsAddress> Addresses;
        public List<TContactContactDetailsLearnerRelationship> LearnerRelationships;
    }
    public class TContactAddressTypeRef : TReferenceClass { }
    public class TContactContactDetailsAddress 
    {
        public Guid ExternalID;
        public Nullable<DateTime> StartDate;
        public Nullable<DateTime> EndDate;
        public TAddress Address;
        public TContactAddressTypeRef AddressType;

    }
    public class TContactLearnerRelationshipTypeRef : TReferenceClass { }
    public class TContactContactDetailsLearnerDetail : TReferenceClass { }
    public class TContactContactDetailsLearnerRelationship
    {
        public Guid ExternalID;
        public Nullable<bool> HasCourtOrder;
        public Nullable<bool> HasParentalResponsibility;
        public TContactLearnerRelationshipTypeRef LearnerContactRelationshipType;
        public Nullable<int> Priority;
        public Nullable<bool> ReceivesCorrespondance;
        public Nullable<bool> ReceivesSchoolReport;
        public TContactContactDetailsLearnerDetail Learner;
    }
    #endregion Contact - Contact Details
    #region Learner Transports
    public class LearnerTransportModeOfTravelRef : TReferenceClass { }
    public class LearnerTransportTravelRouteRef : TReferenceClass { }
    public class TLearnerTransport
    {
        public Guid ExternalID;
        public LearnerTransportModeOfTravelRef ModeOfTravel;
        public LearnerTransportTravelRouteRef TravelRoute;
        public Nullable<bool> LAProvidesTransport;
    }

    #endregion Learner Transports
    #region L
    #endregion L
}
