﻿// ---------------------------------------------------------------------------------
// 
// Sample code is copyright (c) Education Software Solutions Ltd 2021
// 
// Education Software Solutions (Hereafter 'ESS') provide sample code to 
// ESS Technical Integrators (Hereafter 'TIs') under consultancy 
// or as part of our TI programme to assist in the production of applications and services
// to benefit mutual customers.
// 
// Sample code is supplied by the ESS TI Team and may be augmented / changed at any time.
//
// ESS grants non exclusive rights to use the code on an 'as is' basis to contracted TIs 
// for the purpose of interfacing with ESS Products.  Code may be used in whole or in part 
// for the TIs products. Rights are subject to the restrictions below.
//
// This code may not be used for commercial purposes without a current TI agreement   
// in place with ESS.  The use of any sample code may incur liabilities for TI contractual 
// fees.  
//
// If the code is provided to eductational establishments for their own use then further 
// consent from ESS must be obtained before the code can be re-used for commercial gain.
//
// No warranty is provided . This header must be retained within the code and Copyright acknowleged as
// Parts (c) Education Software Solutions Ltd 2021 in version info.
//
// Code samples may be used to support other partners and no exclusivity is implied herein
//
// ---------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Text;

namespace SIMS8V3Models
{
    public class TMaritalStatusRef : TReferenceClass { }
    public class TNationalIdentityRef : TReferenceClass { }
    public class TReasonForChangeRef : TReferenceClass { }
    public class TStaffPreviousName
    {
        public Guid ExternalID;
        public string LegalForename;
        public string LegalMiddleNames;
        public string LegalSurname;
        public Nullable<DateTime> DateOfChange;
        public TReasonForChangeRef ReasonForChange;

    }
    public class TStaffIndentifiers

    {
        /// <summary>
        /// The unique identifier for the Staff
        /// </summary>
        public Guid ExternalID;
        /// <summary>
        /// The known as forename of the Staff
        /// </summary>
        public string Forename;
        /// <summary>
        /// The known as or preferred surname of a Staff
        /// </summary>
        public string Surname;
        public Nullable<bool> IsDeceased;
        public string StaffCode;
        public string Photograph;
        public string GenderCategory;
        public bool? DoNotCommunicate;
        public string WorkTelephone;
        public string WorkEmail;
        public string CurrentStatus;
        public string TeacherReferenceNumber;

    }
   
    public class TStaffNote
    {
        public Guid ExternalID;
        public string Summary;
        public string Attachments;
        public string Note;
        public Nullable<bool> Confidential;

    }
    public class TStaffPersonalExtendeds
    {
        /// <summary>
        /// The unique identifier for the Staff
        /// </summary>
        public Guid ExternalID;
        public string LegalForename;
        public string LegalMiddleNames;
        public string LegalSurname;
        public TMaritalStatusRef MaritalStatus;
        public Nullable<DateTime> DateOfBirth;
        public string QuickNote;
        public TNationalIdentityRef NationalIdentity;
        public List<TStaffPreviousName> PreviousNames;
        public List<TStaffNote> Notes;
    }
    public class TStaffPersonalBasic
    {
        /// <summary>
        /// The unique identifier for the Staff
        /// </summary>
        public Guid ExternalID;
        /// <summary>
        /// The known as forename of the Staff
        /// </summary>
        public string PreferredForename;
        /// <summary>
        /// The known as or preferred surname of a Staff
        /// </summary>
        public string PreferredSurname;
        public Nullable<bool> IsDeceased;
        public string StaffCode;
        public string Photograph;
        public TGenderRef Gender;
        public TTitleRef Title;
    }
    public class TStaffService
    {
        public Guid ExternalID;
        public List<TStaffRoleAssignment> StaffRoleAssignments;
        public List<TStaffServiceRecord> StaffServiceRecords;
    }
    /// <summary>
    /// Service dates at the school
    /// </summary>
    public class TStaffRoleAssignment
    {
        public Guid ExternalID;
        public DateTime? ServiceStartDate;
        public DateTime? ServiceEndDate;
    }
    public class TStaffServiceRecord
    {
        public Guid ExternalID;
        public DateTime? StartDate;
        public DateTime? EndDate;
        public TStaffRoleRef StaffRole;

    }
    //public class TEthnicityRef : TReferenceClass { }
    //public class TReligionRef : TReferenceClass { }
    //public class TLanguageRef : TReferenceClass { }
    public class TProficiencyRef : TReferenceClass { }
    public class TLanguageAssessmentTypeRef : TReferenceClass { }
    public class TStaffLanguageRelationship 
    {
        public TLanguageRef Language;
        public Guid ExternalID;
        public bool? IsFirstLanguage;
        public TProficiencyRef Proficiency;
        public TLanguageAssessmentTypeRef LanguageAssessmentType;
    }
    public class TStaffProfessionals
    {
        public Guid ExternalID;
        public bool? IsTeachingStaff;
        public string TeacherReferenceNumber;
        public List<TStaffQualification> StaffQualifications;
        public List<TStaffProfessionalAccreditation> ProfessionalAccreditations;
        public List<TStaffExperience> StaffExperiences;

    }
    public class TStaffProfessionalAccreditation 
    {
        public Guid ExternalID;
        public DateTime? DateAwarded;
        public TProfessionalAccreditationStatusRef ProfessionalAccreditationStatus;
        public TProfessionalAccreditationRouteRef ProfessionalAccreditationRoute;
        public TProfessionalAccreditationTypeRef ProfessionalAccreditationType;

    }
    public class TProfessionalAccreditationTypeRef : TReferenceClass { }
    public class TProfessionalAccreditationRouteRef : TReferenceClass { }
    public class TProfessionalAccreditationStatusRef : TReferenceClass { }

    public class TStaffQualification 
    {
        public Guid ExternalID;
        public TQualificationLevelRef QualificationLevel;
        public TCountryOfOriginRef Country;
        public TSecondSubjectQualifiedRef SecondSubjectQualified;
        public TFirstSubjectQualifiedRef FirstSubjectQualified;
        public TClassOfDegreeRef ClassOfDegree;
        public bool? Verified;
        public DateTime? DateAwarded;
        public string LocalAuthorityClassification;
        public string Comments;
        public string Title;
    }

    public class TClassOfDegreeRef : TReferenceClass { }
    public class TSecondSubjectQualifiedRef : TReferenceClass { }
    public class TFirstSubjectQualifiedRef : TReferenceClass { }
    public class TCountryOfOriginRef : TReferenceClass { }
    public class TQualificationLevelRef : TReferenceClass { }
    public class TStaffExperience
        {
        public Guid ExternalID;
        public string Note;
        public string ReasonForLeaving;
        public DateTime? DateOfLeaving;
        public DateTime? DateOfArrival;
        public string StaffRole;
        public string Employer;
    }

    public class TStaffCulturals
    {
        public Guid ExternalID;
        public TEthnicityRef Ethnicity;
        public TReligionRef Religion;
        public List<TStaffLanguageRelationship> Languages;
        
    }
    public class TStaffRoleRef : TReferenceClass { }
    public class TStaffServiceExtended
    {
        public Guid ExternalID;
        public string NationalInsuranceNumber;
        public List<TStaffServiceRecordDetails> StaffServiceRecordDetail;
    }
    public class TStaffServiceRecordDetails
    {
        public Guid ExternalID;
        public DateTime? ContinuousServiceStartDate;
        public DateTime? LocalAuthorityStartDate;
        public string PreviousEmployer;
        public string Destination;
        public string NextEmployer;
        public string Notes;
        public TReasonForLeaving ReasonForLeaving;
    }
    public class TReasonForLeaving : TReferenceClass { }
    public class TStaffMedical
    {
        public Guid ExternalID;
        public string DisabilityNumber;
        public TDisabilityIndicator DisabilityIndicator;
        public List<TStaffImpairment> Impairments;
        public List<TStaffMedicalNote> MedicalNotes;
    }
    public class TDisabilityIndicator : TReferenceClass { }
    public class TStaffImpairment : TReferenceClass { }
    public class TStaffMedicalNote : TReferenceClass { }
}




    

